/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.taglib.html;

import java.lang.reflect.InvocationTargetException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.struts.Globals;
import org.apache.struts.taglib.TagUtils;
import org.apache.struts.taglib.html.MultiboxTag;
import org.hdiv.dataComposer.IDataComposer;
import org.hdiv.util.HDIVUtil;

/**
 * <p>
 * Renders an HTML &lt;input&gt; element of type <code>checkbox</code>, whose
 * "checked" status is initialized based on whether the specified value matches one
 * of the elements of the underlying property's array of current values. This element
 * is useful when you have large numbers of checkboxes, and prefer to combine the
 * values into a single array-valued property instead of multiple boolean properties.
 * This tag is only valid when nested inside a form tag body.
 * </p>
 * <p>
 * <strong>WARNING</strong>: In order to correctly recognize cases where none of the
 * associated checkboxes are selected, the <code>ActionForm</code> bean associated
 * with this form must include a statement setting the corresponding array to zero
 * length in the <code>reset()</code> method.
 * </p>
 * <p>
 * The value to be returned to the server, if this checkbox is selected, must be
 * defined by one of the following methods:
 * </p>
 * <ul>
 * <li>Specify a <code>value</code> attribute, whose contents will be used
 * literally as the value to be returned.</li>
 * <li>Specify no <code>value</code> attribute, and the nested body content of
 * this tag will be used as the value to be returned.</li>
 * </ul>
 * 
 * @author Gorka Vicente
 * @see org.apache.struts.taglib.html.MultiboxTag
 */
public class MultiboxTagHDIV extends MultiboxTag {

	/**
	 * Universal version identifier. Deserialization uses this number to ensure that
	 * a loaded class corresponds exactly to a serialized object.
	 */
	private static final long serialVersionUID = -7997128930317769675L;

	/**
	 * Render an input element for this tag.
	 *
	 * @exception JspException if a JSP exception has occurred
	 */
	public int doEndTag() throws JspException {

		// Create an appropriate "input" element based on our parameters
		StringBuffer results = new StringBuffer("<input type=\"checkbox\"");
		
		renderAttribute(results, "name", renderName());
        renderAttribute(results, "accesskey", accesskey);
        renderAttribute(results, "tabindex", tabindex);
		
        String value = renderValue(results);
        prepareChecked(results, value);

		results.append(prepareEventHandlers());
		results.append(prepareStyles());
		results.append(getElementClose());

		// Render this element to our response
		TagUtils.getInstance().write(pageContext, results.toString());

		// Continue evaluating this page
		return (EVAL_PAGE);
	}
	
    /**
	 * Prepare the name element
	 * 
	 * @return The element name.
	 */
    protected String renderName() throws JspException {
        return property;
    }	

    /**
	 * Render the value element
	 * 
	 * @param results The StringBuffer that output will be appended to.
	 * @see org.hdiv.composer.IDataComposer#compose(String, String, boolean)
	 */
    protected String renderValue(StringBuffer results) throws JspException {

        String value = (this.value == null) ? this.constant : this.value;
        if (value == null) {
            JspException e = new JspException(messages.getMessage("multiboxTag.value"));
            pageContext.setAttribute(Globals.EXCEPTION_KEY, e, PageContext.REQUEST_SCOPE);
            throw e;
        }

        IDataComposer dataComposer = HDIVUtil.getDataComposer();
		String cipheredValue = dataComposer.compose(property, value, false);
        
        renderAttribute(results, "value", TagUtils.getInstance().filter(cipheredValue));
        
        // returns unciphered value to check selected options
        return value;
    }
    
    /**
	 * Render the checked element
	 * 
	 * @param results The StringBuffer that output will be appended to.
	 */
    protected void prepareChecked(StringBuffer results, String value) throws JspException {

        Object bean = TagUtils.getInstance().lookup(pageContext, name, null);
        
        String values[] = null;        
        if (bean == null) {
            throw new JspException(messages.getMessage("getter.bean", name));
        }
            
        try {
            values = BeanUtils.getArrayProperty(bean, property);
            if (values == null) {
                values = new String[0];
            }
                
        } catch (IllegalAccessException e) {
            throw new JspException(messages.getMessage("getter.access", property, name));
        } catch (InvocationTargetException e) {
            Throwable t = e.getTargetException();
            throw new JspException(messages.getMessage("getter.result", property, t.toString()));
        } catch (NoSuchMethodException e) {
            throw new JspException(messages.getMessage("getter.method", property, name));
        }
        
        for (int i = 0; i < values.length; i++) {
            if (value.equals(values[i])) {            	
                results.append(" checked=\"checked\"");
                break;
            }
        }
    }    
    
	/**
	 * Prepares an attribute if the value is not null, appending it to the the given
	 * StringBuffer.
	 * @param handlers The StringBuffer that output will be appended to.
	 */
	protected void renderAttribute(StringBuffer handlers, String name, Object value) {

		if (value != null) {
			handlers.append(" ");
			handlers.append(name);
			handlers.append("=\"");
			handlers.append(value);
			handlers.append("\"");
		}
	}    
}
